##
# * prod_hostinspection
# *
# * Copyright (c) 2015 Ferrari electronic AG

'use strict'

spawn = require "child_process"
  .spawn

appConfig = global.appConfig
runAsRoot = appConfig.sudo or false if appConfig

debug = require("debug")("omg-hostinspection")

# @class Producer
Producer = require "../../ProducerClass"

# Hostinspection
# @class Hostinspection
# @extends Producer
class Hostinspection extends Producer
  # isRunning indicates if hostinspection is currently running a task
  # @type {Boolean}
  isRunning = false

  ##
  # Constructor
  constructor: ->
    #call the producer super class and provide a name
    super "hostinspection"

    # Run the given command as a spawn child and publish the returned data
    # @private
    # @param {String} command to execute
    # @param {Array} arguments handed to the command
    run = (command, args) =>
      @sendStatus "running"
      runningCommand = spawn command, args
      runningCommand.stdout.on "data", (data) =>
        lines = data.toString('utf-8')
        lines = lines.split('\n')
        lines.pop()
        # debug "#{command} produced: #{lines}"
        @sendData lines, true

      #log stderr
      runningCommand.stderr.on "data", (data) ->
        debug "data #{data}"

      #log errors occuring on the spawn child
      runningCommand.on "error", (e)->
        debug e

      #notify subscribers that the spawned process was closed
      runningCommand.on "close", (returnCode) =>
        @sendStatus "stopped"

    ##
    # runs "ping"
    # @public
    # @param {String} hostname
    @ping = (host) =>
      command = "ping"
      host = host || "127.0.0.1"
      args = ["-c", "4", host]
      run command, args

    ##
    # runs "tracepath"
    # @public
    # @param {String} hostname
    @tracepath = (host) =>
      command = "tracepath"
      host = host || "127.0.0.1"
      args = [host]
      run command, args

    ##
    # runs "dig" dns lookup
    # @public
    # @param {String} hostname
    @dnslookup = (host) =>
      command = "dig"
      host = host || "127.0.0.1"
      args = [host]
      run command, args

    ##
    # runs "nmap"
    # @public
    # @param {String} hostname
    @portscan = (host) =>
      host = host || "127.0.0.1"
      command = ""
      if runAsRoot
        command = "sudo"
        args = ["nmap", "-PO", "-vv"]

      unless runAsRoot
        command = "nmap"
        args = ["-PO", "-vv"]

      args.push "tcp"
      args.push host
      # args.push "tcp" if proto is "tcp"
      # args.push "-sU" if proto is "udp"
      run command, args


    ##
    # runs "nmap" on specified port with "-sT" or "-sU"
    # @public
    # @param {String} hostname
    # @param {String} protocol
    # @param {String} port
    @portprobe = (host, proto, port) =>
      unless host and proto and port
        debug "host, protocol and port have to be specified"
        return

      command = ""

      if runAsRoot
        command = "sudo"
        args = ["nmap", "-PO", "-vv"]
      if not runAsRoot
        command = "nmap"
        args = ["-PO", "-vv"]

      args.push "-sT" if proto is "tcp"
      args.push "-sU" if proto is "udp"
      args.push host
      args.push "-p"
      args.push port
      run command, args


module.exports = new Hostinspection()